@@ -23,18 +23,31 @@
 #include <gwenhywfar/logger.h>
 #include <gwenhywfar/debug.h>
 #include <aqbanking/banking.h>
-
+#include <aqbanking/abgui.h>
 
 #include "globals.h"
 
-#ifdef WITH_STATIC_PLUGINS
-int GWEN_Plugins_Init();
-int AB_Plugins_Init();
-int LC_Plugins_Init();
-#ifdef ENABLE_EBICS_PLUGINS
-int EBC_Plugins_Init();
-#endif
-#endif
+
+
+static void showVersions() {
+  int vmajor, vminor, vpatchLevel, vbuild;
+
+  fprintf(stdout, "Versions:\n");
+  GWEN_Version(&vmajor,
+	       &vminor,
+	       &vpatchLevel,
+	       &vbuild);
+  fprintf(stdout, " Gwenhywfar   : %d.%d.%d.%d\n",
+	  vmajor, vminor, vpatchLevel, vbuild);
+
+  AB_Banking_GetVersion(&vmajor,
+			&vminor,
+			&vpatchLevel,
+			&vbuild);
+  fprintf(stdout, " AqBanking    : %d.%d.%d.%d\n",
+	  vmajor, vminor, vpatchLevel, vbuild);
+
+}
 
 
 
@@ -44,6 +57,7 @@ int main(int argc, char **argv) {
   const char *cmd;
   const char *pinFile;
   int nonInteractive=0;
+  int acceptValidCerts=0;
   int rv;
   AB_BANKING *ab;
   GWEN_GUI *gui;
@@ -83,6 +97,17 @@ int main(int argc, char **argv) {
     "This automatically returns a confirmative answer to any non-critical\n"
     "message."
   },
+  {
+    0,                            /* flags */
+    GWEN_ArgsType_Int,            /* type */
+    "acceptValidCerts",           /* name */
+    0,                            /* minnum */
+    1,                            /* maxnum */
+    "A",                          /* short option */
+    "acceptvalidcerts",           /* long option */
+    "Automatically accept all valid TLS certificate",
+    "Automatically accept all valid TLS certificate"
+  },
   {
     GWEN_ARGS_FLAGS_HAS_ARGUMENT, /* flags */
     GWEN_ArgsType_Char,           /* type */
@@ -127,6 +152,7 @@ int main(int argc, char **argv) {
 		     db);
   if (rv==GWEN_ARGS_RESULT_ERROR) {
     fprintf(stderr, "ERROR: Could not parse arguments main\n");
+    GWEN_DB_Group_free(db);
     return 1;
   }
   else if (rv==GWEN_ARGS_RESULT_HELP) {
@@ -174,6 +200,10 @@ int main(int argc, char **argv) {
                              I18N("  getaccounts:\n"
                                   "    Requests account list for a "
                                   "user\n\n"));
+    GWEN_Buffer_AppendString(ubuf,
+                             I18N("  getaccsepa:\n"
+                                  "    Requests SEPA account list for a "
+                                  "user\n\n"));
     GWEN_Buffer_AppendString(ubuf,
                              I18N("  getsysid:\n"
                                   "    Requests a system id for the given "
@@ -198,9 +228,29 @@ int main(int argc, char **argv) {
     GWEN_Buffer_AppendString(ubuf,
                              I18N("  addaccount:\n"
                                   "    Manually add account \n\n"));
+    GWEN_Buffer_AppendString(ubuf,
+                             I18N("  delaccount:\n"
+                                  "    Deletes account \n\n"));
+
+    GWEN_Buffer_AppendString(ubuf,
+                             I18N("  sethbciversion:\n"
+                                  "    Set the HBCI protocol version to be used\n\n"));
+
+    GWEN_Buffer_AppendString(ubuf,
+                             I18N("  setMaxTransfers:\n"
+                                  "    Set the maximum number of transfers/debit notes per job \n\n"));
+
+    GWEN_Buffer_AppendString(ubuf,
+                             I18N("  setsepaprofile:\n"
+                                  "    Set the SEPA profile for transfers/debit notes\n\n"));
+
+    GWEN_Buffer_AppendString(ubuf,
+                             I18N("  setTanMediumId:\n"
+                                  "    Set the medium id for some PIN/TAN methods (like mTAN) \n\n"));
 
     fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
     GWEN_Buffer_free(ubuf);
+    GWEN_DB_Group_free(db);
     return 0;
   }
   if (rv) {
@@ -208,44 +258,30 @@ int main(int argc, char **argv) {
     argv+=rv-1;
   }
 
-#ifdef WITH_STATIC_PLUGINS
-  rv=GWEN_Plugins_Init();
-  if (rv<0) {
-    fprintf(stderr, "ERROR: Unable to init GWEN plugins (%d).\n", rv);
-    return 1;
-  }
-  rv=LC_Plugins_Init();
-  if (rv<0) {
-    fprintf(stderr, "ERROR: Unable to init Libchipcard plugins (%d).\n", rv);
-    return 1;
-  }
-  rv=AB_Plugins_Init();
-  if (rv<0) {
-    fprintf(stderr, "ERROR: Unable to init AqBanking plugins (%d).\n", rv);
-    return 1;
-  }
-
-#ifdef ENABLE_EBICS_PLUGINS
-  rv=EBC_Plugins_Init();
-  if (rv<0) {
-    fprintf(stderr, "ERROR: Unable to init AqEBICS plugins (%d).\n", rv);
-    return 1;
-  }
-#endif
-#endif
-
   /*GWEN_Logger_SetLevel(AQHBCI_LOGDOMAIN, GWEN_LoggerLevelInfo); */
   cmd=GWEN_DB_GetCharValue(db, "params", 0, 0);
   if (!cmd) {
     fprintf(stderr, "ERROR: Command needed.\n");
+    GWEN_DB_Group_free(db);
     return 1;
   }
 
   gui=GWEN_Gui_CGui_new();
-  s=GWEN_DB_GetCharValue(db, "charset", 0, "ISO-8859-15");
-  GWEN_Gui_CGui_SetCharSet(gui, s);
+  s=GWEN_DB_GetCharValue(db, "charset", 0, NULL);
+  if (s && *s)
+    GWEN_Gui_SetCharSet(gui, s);
   nonInteractive=GWEN_DB_GetIntValue(db, "nonInteractive", 0, 0);
-  GWEN_Gui_CGui_SetIsNonInteractive(gui, nonInteractive);
+  acceptValidCerts=GWEN_DB_GetIntValue(db, "acceptValidCerts", 0, 0);
+  if (nonInteractive)
+    GWEN_Gui_AddFlags(gui, GWEN_GUI_FLAGS_NONINTERACTIVE);
+  else
+    GWEN_Gui_SubFlags(gui, GWEN_GUI_FLAGS_NONINTERACTIVE);
+
+  if (acceptValidCerts)
+    GWEN_Gui_AddFlags(gui, GWEN_GUI_FLAGS_ACCEPTVALIDCERTS);
+  else
+    GWEN_Gui_SubFlags(gui, GWEN_GUI_FLAGS_ACCEPTVALIDCERTS);
+
   pinFile=GWEN_DB_GetCharValue(db, "pinFile", 0, NULL);
   if (pinFile) {
     GWEN_DB_NODE *dbPins;
@@ -253,17 +289,22 @@ int main(int argc, char **argv) {
     dbPins=GWEN_DB_Group_new("pins");
     if (GWEN_DB_ReadFile(dbPins, pinFile,
 			 GWEN_DB_FLAGS_DEFAULT |
-			 GWEN_PATH_FLAGS_CREATE_GROUP,
-			 0, 20000)) {
+			 GWEN_PATH_FLAGS_CREATE_GROUP)) {
       fprintf(stderr, "Error reading pinfile \"%s\"\n", pinFile);
+      GWEN_DB_Group_free(dbPins);
+      GWEN_DB_Group_free(db);
       return 2;
     }
-    GWEN_Gui_CGui_SetPasswordDb(gui, dbPins, 1);
+    GWEN_Gui_SetPasswordDb(gui, dbPins, 1);
   }
   GWEN_Gui_SetGui(gui);
 
-  ab=AB_Banking_new("aqhbci-tool", GWEN_DB_GetCharValue(db, "cfgfile", 0, 0),
-		    0);
+  ab=AB_Banking_new("aqhbci-tool", GWEN_DB_GetCharValue(db, "cfgfile", 0, 0), 0);
+
+  AB_Banking_RuntimeConfig_SetCharValue(ab, "fintsRegistrationKey", "32F8A67FE34B57AB8D7E4FE70");
+  AB_Banking_RuntimeConfig_SetCharValue(ab, "fintsApplicationVersionString", AQBANKING_FINTS_VERSION_STRING);
+
+  AB_Gui_Extend(gui, ab);
 
   if (strcasecmp(cmd, "mkpinlist")==0) {
     rv=mkPinList(ab, db, argc, argv);
@@ -280,6 +321,9 @@ int main(int argc, char **argv) {
   else if (strcasecmp(cmd, "addaccount")==0) {
     rv=addAccount(ab, db, argc, argv);
   }
+  else if (strcasecmp(cmd, "delaccount")==0) {
+    rv=delAccount(ab, db, argc, argv);
+  }
   else if (strcasecmp(cmd, "getsysid")==0) {
     rv=getSysId(ab, db, argc, argv);
   }
@@ -307,6 +351,12 @@ int main(int argc, char **argv) {
   else if (strcasecmp(cmd, "getitanmodes")==0) {
     rv=getItanModes(ab, db, argc, argv);
   }
+  else if (strcasecmp(cmd, "listitanmodes")==0) {
+    rv=listItanModes(ab, db, argc, argv);
+  }
+  else if (strcasecmp(cmd, "setitanmode")==0) {
+    rv=setItanMode(ab, db, argc, argv);
+  }
   else if (strcasecmp(cmd, "changepin")==0) {
     rv=changePin(ab, db, argc, argv);
   }
@@ -325,6 +375,25 @@ int main(int argc, char **argv) {
   else if (strcasecmp(cmd, "subaccountflags")==0) {
     rv=addsubAccountFlags(ab, db, argc, argv, 0);
   }
+  else if (strcasecmp(cmd, "setmaxtransfers")==0) {
+    rv=setMaxTransfers(ab, db, argc, argv);
+  }
+  else if (strcasecmp(cmd, "setTanMediumId")==0) {
+    rv=setTanMediumId(ab, db, argc, argv);
+  }
+  else if (strcasecmp(cmd, "logfile")==0) {
+    rv=logFile(ab, db, argc, argv);
+  }
+  else if (strcasecmp(cmd, "getaccsepa")==0) {
+    rv=getAccSepa(ab, db, argc, argv);
+  }
+  else if (strcasecmp(cmd, "setsepaprofile")==0) {
+    rv=setSepaProfile(ab, db, argc, argv);
+  }
+  else if (strcasecmp(cmd, "versions")==0) {
+    showVersions();
+    rv=0;
+  }
   else if (strcasecmp(cmd, "test1")==0) {
     rv=test1(ab, db, argc, argv);
   }
@@ -334,6 +403,7 @@ int main(int argc, char **argv) {
     rv=1;
   }
 
+  GWEN_DB_Group_free(db);
   return rv;
 }
 
